/* Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"

// Module Name: rtavr_intcont
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                          DO:  NORMAL    TRISTATE(FAKE)   
// Number of Slice Flip Flops:     33
// Number of 4 input LUTs:         54
// Number of occupied Slices:      42
// Total Number of 4 input LUTs:   62
// Number of bonded IOBs :         29
//   IOB Flip Flops :               -
// Number of BUFGMUXs :             1          1
// Number of RAMB16BWEs :           -          -

module rtavr_intcont(
		input CLK,
		input RESET,

		output [4:0] INT_VEC,
		output INT_REQ,
		input INT_ACK

		, input IOR_INT0
		, output IOR_RST_INT0

`ifdef IOR_HAVE_TIM0
		, input IOR_TIM0_OVF
		, input IOR_TIM0_COMPA
		, input IOR_TIM0_COMPB
		, output IOR_RST_TIM0_OVF
		, output IOR_RST_TIM0_COMPA
		, output IOR_RST_TIM0_COMPB
`endif
`ifdef IOR_HAVE_SPI
		, input IOR_SPI_INT
		, output IOR_RST_SPI_INT
`endif
`ifdef IOR_HAVE_USART
		, input IOR_USART_TX
		, input IOR_USART_UDRE
		, input IOR_USART_RX
		, output IOR_RST_USART_TX
		, output IOR_RST_USART_UDRE
		, output IOR_RST_USART_RX
`endif
		);

//`define INT0_vect_num  	 1
//`define TIM0_COMPA_vect_num  	11
//`define TIM0_COMPB_vect_num  	12
//`define TIM0_OVF_vect_num  	13
//`define USART_TX_vect_num     14
//`define USART_UDRE_vect_num   15
//`define TWI_SLAVE_vect_num	16
//`define SPI_vect_num  	17
//`define USART_RX_vect_num     18

  reg [4:0] r_int_vec;  

  wire [4:0] v_int_vec =   IOR_INT0 ?  `INT0_vect_num
`ifdef IOR_HAVE_TIM0
                  :  IOR_TIM0_COMPA ? `TIM0_COMPA_vect_num
                  :  IOR_TIM0_COMPB ? `TIM0_COMPB_vect_num
                  :  IOR_TIM0_OVF   ? `TIM0_OVF_vect_num
`endif
`ifdef IOR_HAVE_USART
                  :  IOR_USART_TX   ? `USART_TX_vect_num
                  :  IOR_USART_UDRE ? `USART_UDRE_vect_num
`endif
`ifdef IOR_HAVE_TWI
                  :  IOR_TWI_SLAVE  ? `TWI_SLAVE_vect_num
`endif
`ifdef IOR_HAVE_SPI
                  :  IOR_SPI_INT    ?  `SPI_vect_num
`endif
`ifdef IOR_HAVE_USART
                  :  IOR_USART_RX   ? `USART_RX_vect_num
`endif
		  : 0;
  assign INT_REQ = I_out & (IOR_INT0 
`ifdef IOR_HAVE_TIM0
                           | IOR_TIM0_OVF | IOR_TIM0_COMPA | IOR_TIM0_COMPB
`endif
`ifdef IOR_HAVE_SPI
			   | IOR_SPI_INT
`endif
`ifdef IOR_HAVE_USART
			   | IOR_RST_USART_TX
`endif
`ifdef IOR_HAVE_TWI
			   | IOR_TWI_SLAVE
`endif
			   );
  assign INT_VEC = r_int_vec;

  assign IOR_RST_INT0        = INT_ACK & (r_int_vec == `INT0_vect_num);
`ifdef IOR_HAVE_TIM0
  assign IOR_RST_TIM0_COMPA  = INT_ACK & (r_int_vec == `TIM0_COMPA_vect_num);
  assign IOR_RST_TIM0_COMPB  = INT_ACK & (r_int_vec == `TIM0_COMPB_vect_num);
  assign IOR_RST_TIM0_OVF    = INT_ACK & (r_int_vec == `TIM0_OVF_vect_num);
`endif
`ifdef IOR_HAVE_USART
  assign IOR_RST_USART_TX    = INT_ACK & (r_int_vec == `USART_TX_vect_num);
`endif
`ifdef IOR_HAVE_TWI
  assign IOR_RST_TWI_SLAVE   = INT_ACK & (r_int_vec == `TWI_SLAVE_vect_num);
`endif
`ifdef IOR_HAVE_SPI
  assign IOR_RST_SPI_INT     = INT_ACK & (r_int_vec == `SPI_vect_num);
`endif

always @(negedge CLK)
  begin
    if (RESET == 0)
      begin
	r_int_vec <= 0;
      end
    else
      begin
        if (~INT_ACK) r_int_vec <= v_int_vec;
      end
  end

endmodule
