/* Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"
// global defines : OUTPUT_MUX_TRI

//`define OUTPUT_MUX_TRI

// Module Name: rtavr_ior_usart
//   version 005
//
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
// SUPPORTED FORMAT (TX): 
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | 1  |                   7N1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | 1  | 1  |              7N2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | P  | 1  |              7P1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | 1  |              8N1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | P  | 1  | 1  |         7P2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | 1  | 1  |         8P2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | P  | 1  |         8P1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | D8 | 1  |         9N1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | P  | 1  | 1  |    8P2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | D8 | 1  | 1  |    9N2
//
//
//   version 004
//                               (FULL)  (1)    (2)   (3)
// Number of Slice Flip Flops:    114     95    87     83
// Number of 4 input LUTs:        177    112   105     98
// Number of occupied Slices:     138    104    98     92
// Total Number of 4 input LUTs:  188    123   112    109
//   version 005
//                               (FULL)  (1)    (2)   (3)
// Number of Slice Flip Flops:    114     95    87     83
// Number of 4 input LUTs:        177    112   108     98
// Number of occupied Slices:     138    104   100     92
// Total Number of 4 input LUTs:  188    123   115    109
// Number of bonded IOBs :         37     35    35     35
//   IOB Flip Flops :               1      1     1      1
// Number of BUFGMUXs :             1 
// Number of RAMB16BWEs :          - 
//
// (1) 8N1 with UBRR (U2X=1)
// (2) 8N1 with UBRR_BITS(8)
// (3) 8N1 with FIXED_UBRR(107) (U2X=1)
//
module rtavr_ior_usart(
		input CLK,
		input RESET,

		input [7:0] DI
`ifdef IOR_LARGE_MAPPER
		, input WE_ubrrl
		, input WE_ubrrh
		, input WE_ucsrc
		, input WE_ucsrb
		, input WE_ucsra
		, input WE_udr
		, input OE_udr
		, output [7:0] DO_ubrrl
		, output [7:0] DO_ubrrh
		, output [7:0] DO_ucsrc
		, output [7:0] DO_ucsrb
		, output [7:0] DO_ucsra
		, output [7:0] DO_udr
`else
		, input WR,RD
		, input [5:0] ADDR
		, output [7:0] DO
`endif
		// external I/O
		, output TX_OUT
		, input RX_IN
		, input XCK_IN
		, input XCK_DDR
		, output XCK_OUT

		, output TXEN
		, output XCKEN
		// interrupt
		, output TX_INT
		, output UDRE_INT
		, output RX_INT
		, input RST_TX_INT
		);
  parameter BASE_ADDR = 6'h0e;  // USART : (0x0e - 0x14 ADC space)
				//  UBRRL, UBRRH, UCSRC, UCSRB, UCSRA, UDR

`ifdef FIXED_UBRR
  reg  [11:0] r_ps;
`elsif UBRR_BITS
  reg  [`UBRR_BITS-1:0] r_ps;
`else
  reg  [11:0] r_ps;
`endif
  reg  tx_mclk;

  reg  [3:0] tx_ps;
  reg  [4:0] tx_count;
  reg  [1:0] r_count_max_x;
  wire [4:0] v_count_max = { 2'b10, r_count_max_x , 1'b1};
  reg  tx_count_en;
  reg  [9:0] tx_data;
  reg  tx_data_load;
  reg  r_txd;

  reg  [3:0] rx_ps;
  reg  [4:0] rx_count;
  reg  rx_count_en;
  reg  [10:0] rx_data;
  reg  rx_data_rdy;
  reg  rx_udr_rdy;
  reg  rx_parity;
  reg  rx_c;

  reg  [7:0] r_udr_tx;
  reg  [7:0] r_udr_rx;
`ifdef FIXED_UBRR
  wire [11:0] UBRR = `FIXED_UBRR ;
`elsif UBRR_BITS
  reg  [`UBRR_BITS-1:0] UBRR;
  wire [15:0] v_ubrr;
`else
  reg  [11:0] UBRR;
`endif

  reg  RXC ;
  reg  TXC;
  wire UDRE;
  reg  FE ;
  reg  DOR;
  reg  UPE ;
`ifdef FIXED_U2X
  wire U2X = `FIXED_U2X ;
`else
  reg  U2X;
`endif
`ifdef FIXED_MPCM
  wire MPCM = `FIXED_MPCM ;
`else
  reg  MPCM;
`endif

  reg  RXCIE;
  reg  TXCIE;
  reg  UDRIE;
  reg  rx_en ;
  reg  tx_en;
  reg  RXB8;
  reg  TXB8;

`ifdef FIXED_UMSEL
  wire [1:0] UMSEL = `FIXED_UMSEL ;
`else
  reg  [1:0] UMSEL;
`endif
`ifdef FIXED_UPM
  wire [1:0] UPM = `FIXED_UPM ;
`else
  reg  [1:0] UPM;
`endif
`ifdef FIXED_USBS
  wire USBS = `FIXED_USBS ;
`else
  reg  USBS;
`endif
`ifdef FIXED_UCSZ
  wire [2:0] UCSZ = `FIXED_UCSZ ;
  wire r_ucsz0 = UCSZ[0];
  wire r_ucsz2 = UCSZ[2];
`else
  reg  r_ucsz2;
  reg  r_ucsz0;
//  wire UCSZ = { r_ucsz2, 1'b1, r_ucsz0 };
`endif
`ifdef FIXED_UCPOL
  wire UCPOL = `FIXED_UCPOL ;
`else
  reg  UCPOL;
`endif

`ifdef IOR_LARGE_MAPPER
  `ifdef FIXED_UBRR
      assign DO_ubrrl[7:0]  = UBRR[7:0];
      assign DO_ubrrh[7:0]  = { 4'h0 , UBRR[11:8] };
  `elsif UBRR_BITS
      assign DO_ubrrl[7:0]  = v_ubrr[7:0];
      assign DO_ubrrh[7:0]  = v_ubrr[15:8];
  `else
      assign DO_ubrrl[7:0]  = UBRR[7:0];
      assign DO_ubrrh[7:0]  = { 4'h0 , UBRR[11:8]};
  `endif
      assign DO_ucsrc[7:0]  = { UMSEL, UPM[1:0], USBS, 1'b1 , r_ucsz0, UCPOL};
      assign DO_ucsrb[7:0]  = {RXCIE, TXCIE, UDRIE, rx_en,
				  tx_en, r_ucsz2 , RXB8, TXB8 };
      assign DO_ucsra[7:0]  = { RXC, TXC, UDRE, FE, DOR, UPE, U2X, MPCM};
      assign DO_udr[7:0]    = r_udr_rx[7:0];
`else // IOR_LARGE_MAPPER
  wire OE_ubrrl = RD & (ADDR == BASE_ADDR);
  wire WE_ubrrl = WR & (ADDR == BASE_ADDR);
  wire OE_ubrrh = RD & (ADDR == (BASE_ADDR + 1)); 
  wire WE_ubrrh = WR & (ADDR == (BASE_ADDR + 1));
  wire OE_ucsrc = RD & (ADDR == (BASE_ADDR + 2)); 
  wire WE_ucsrc = WR & (ADDR == (BASE_ADDR + 2));
  wire OE_ucsrb = RD & (ADDR == (BASE_ADDR + 3)); 
  wire WE_ucsrb = WR & (ADDR == (BASE_ADDR + 3));
  wire OE_ucsra = RD & (ADDR == (BASE_ADDR + 4)); 
  wire WE_ucsra = WR & (ADDR == (BASE_ADDR + 4));
  wire OE_udr   = RD & (ADDR == (BASE_ADDR + 5)); 
  wire WE_udr   = WR & (ADDR == (BASE_ADDR + 5));

  wire [7:0] i_DO = 
`ifdef FIXED_UBRR
		OE_ubrrl ? UBRR[7:0] 
		: OE_ubrrh ? { 4'h0 , UBRR[11:8] }
`elsif UBRR_BITS
		OE_ubrrl ? v_ubrr[7:0] 
		: OE_ubrrh ? v_ubrr[15:8] 
`else
		OE_ubrrl ? UBRR[7:0] 
		: OE_ubrrh ? { 4'h0 , UBRR[11:8] }
`endif

		: OE_ucsra ? { RXC, TXC, UDRE, FE, DOR, UPE, U2X, MPCM}
		: OE_ucsrb ?  {RXCIE, TXCIE, UDRIE, rx_en,
				  tx_en, r_ucsz2 , RXB8, TXB8 }
		: OE_ucsrc ? { UMSEL, UPM[1:0], USBS, 1'b1 , r_ucsz0, UCPOL}
`ifdef OUTPUT_MUX_OR
		: OE_udr ? r_udr_rx : 0;
`else
		: r_udr_rx;
`endif

`ifdef OUTPUT_MUX_TRI
  bufif1(DO, i_DO, (OE_ubrrl | OE_ubrrh | OE_ucsrb| OE_ucsra | OE_udr));
`else
  assign DO = i_DO;
`endif
`endif // IOR_LARGE_MAPPER

  assign UDRE	  = ~(tx_data_load);
  assign TX_INT   = TXC  & TXCIE;
  assign RX_INT   = RXC  & RXCIE;
  assign UDRE_INT = UDRE & UDRIE;
  assign TX_OUT   = UMSEL[1] ? tx_data[0] : r_txd;
  assign XCK_OUT  = tx_mclk;
  assign XCKEN    = (tx_en | rx_en) & UMSEL[0];
  assign TXEN	  = tx_en;

  wire SPI_MODE	  = UMSEL[1];

  // v_tclk/r_tclk  : internal clock for tx
  wire v_tclk =  (SPI_MODE & XCK_DDR ) ? tx_ps[0]
		 : (UMSEL[0] & ~XCK_DDR ) ? ( XCK_IN ^ UCPOL)
		 : (U2X == 1) ? tx_ps[2]
		 : tx_ps[3] ;
  reg  r_tclk;
 
  wire do_txcount = tx_count_en & (r_tclk ^ v_tclk);
  wire do_txin  =  do_txcount &  r_tclk;
  wire do_txsft =  do_txcount & ~r_tclk;


  wire v_bit7 = (~SPI_MODE & ~r_ucsz0);
  wire v_bit8 = ( SPI_MODE | &r_ucsz0 & ~r_ucsz2);
  wire v_bit9 = (~SPI_MODE & r_ucsz2);
 
  wire [1:0] v_bit_num_x = SPI_MODE ? 0
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | 1  |                   7N1
			 : (v_bit7 & ~UPM[1] & ~USBS) ? 0
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | 1  | 1  |              7N2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | P  | 1  |              7P1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | 1  |              8N1
  			 :  (v_bit7 & (UPM[1] ^ USBS))
  			   |(v_bit8 & ~UPM[1] & ~USBS) ? 1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | P  | 1  | 1  |         7P2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | 1  | 1  |         8P2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | P  | 1  |         8P1
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | D8 | 1  |         9N1
  			 :  (v_bit7 & UPM[1] & USBS)
  			   |(v_bit8 & (UPM[1] ^ USBS))
  			   |(v_bit9 & ~UPM[1] & ~USBS) ? 2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | P  | 1  | 1  |    8P2
// | 0 | D0 | D1 | D2 | D3 | D4 | D5 | D6 | D7 | D8 | 1  | 1  |    9N2
  			 :  3 ;

  wire tx_parity7 = r_udr_tx[0] ^ r_udr_tx[1] ^ r_udr_tx[2] ^ r_udr_tx[3]
		   ^ r_udr_tx[4] ^ r_udr_tx[5] ^ r_udr_tx[6];
  wire tx_parity8 = tx_parity7 ^ r_udr_tx[7];


`ifdef USART_LOCAL_LB
  wire v_rxd = r_txd;
`else
  wire v_rxd = RX_IN;
`endif

  // v_rclk/r_rclk  : internal clock for rx
  wire v_rclk = (UMSEL[0] & ~XCK_DDR ) ? ( XCK_IN ^ UCPOL )
  		 : (UMSEL[0] & XCK_DDR ) ? ( XCK_OUT ^ UCPOL )
		 : (U2X == 1) ? ~rx_ps[2]
		 : ~rx_ps[3] ;
  reg  r_rclk;

  wire do_rx = ~(WE_ucsrb & ~DI[4]) & rx_en & (r_rclk ^ v_rclk);
  wire do_rxcount = do_rx & (rx_count_en | SPI_MODE);
  wire do_rxin  =  do_rxcount & ( r_rclk ^ SPI_MODE);
  wire do_rxsft =  do_rxcount & (~r_rclk ^ SPI_MODE);

  always @(negedge CLK)
  begin
    if (RESET == 0)
      begin
      end
    else
      begin
      end
  end

  always @(posedge CLK)
  begin
    if (RESET == 0)
      begin
	r_ps <= 0;
	r_tclk <= 0;
	r_rclk <= 1;
	tx_mclk <= 0;

	tx_ps <= 0;
	tx_data_load <= 0;
	tx_count <= 0;
	r_count_max_x <= 0;
	tx_count_en <= 0;
	tx_data <= 10'b1111111111;
	r_txd <= 1;

	rx_ps <= 0;
	rx_count <= 0;
	rx_count_en <= 0;
	rx_data <= 0;
	rx_data_rdy <= 0;
	rx_udr_rdy <= 0;

`ifdef FIXED_UBRR
`elsif UBRR_BITS
`else
	UBRR <= 0;
`endif
	r_udr_rx <= 0;
	r_udr_tx <= 0;

	RXC  <= 0;
	TXC <= 0;
	FE  <= 0;
	DOR <= 0;
	UPE  <= 0;
`ifdef FIXED_U2X
`else
	U2X <= 0;
`endif
`ifdef FIXED_MPCM
`else
	MPCM <= 0;
`endif

	RXCIE  <= 0;
	TXCIE  <= 0;
	UDRIE  <= 0;
	rx_en  <= 0;
	tx_en  <= 0;
	RXB8 <= 0;
	TXB8 <= 0;

`ifdef FIXED_UMSEL
`else
	UMSEL <= 0;
`endif
`ifdef FIXED_UPM
`else
	UPM <= 0;
`endif
`ifdef FIXED_USBS
`else
	USBS <= 0;
`endif
`ifdef FIXED_UCSZ
`else
	r_ucsz0 <= 0;
	r_ucsz2 <= 0;
`endif
`ifdef FIXED_UCPOL
`else
	UCPOL <= 0;
`endif
      end
    else
      begin
	r_tclk <= v_tclk;

	if (r_ps == UBRR) r_ps <= 0;
	else r_ps <= r_ps + 1;

	if(WE_udr)
	  begin
	    r_udr_tx <= DI;
	    tx_data_load <= tx_en;
	  end

	if (tx_data_load & ~tx_count_en & ~r_tclk)
	  begin
	    tx_data_load <= 0;
//	    tx_ps <= 0;
	    tx_count <= SPI_MODE ? 5'b00001 : 5'b11110;
	    tx_count_en <= 1;
	    r_count_max_x <= v_bit_num_x;

	    tx_data[0] <= 0; // start bit
	    tx_data[7:1] <= r_udr_tx[6:0]; 
	    tx_data[8] <= (~v_bit7) ? r_udr_tx[7]
			 : (v_bit7 & UPM[1]) ? (UPM[0] ^ tx_parity7)
			 : 1;
	    tx_data[9] <= (v_bit9) ? TXB8
			 : (v_bit8 & UPM[1]) ? (UPM[0] ^ tx_parity8)
			 : 1;
	  end
//        else if (tx_count_en & (r_ps == UBRR))
         if (r_ps == UBRR)
	    tx_ps <= tx_ps + 1;

	if (do_txcount)
	  begin
	    tx_count <= tx_count + 1;
	    if (tx_count == v_count_max) tx_count_en <= 0;
	  end
	if (do_txsft)
	  begin
	    r_txd <= tx_data[0];
	    tx_data[6:0] <= tx_data[7:1];
	    tx_data[7]  <= (~v_bit7) | (v_bit7 & UPM[1]) ? tx_data[8] : 1;
	    tx_data[8]  <= ( v_bit9) | (v_bit8 & UPM[1]) ? tx_data[9] : 1;
	    tx_data[9]  <= 1;

	    if (SPI_MODE) tx_mclk <= UCPOL;
	  end
	if (do_txin)
	  begin
	    if (SPI_MODE) tx_mclk <= ~UCPOL;
	  end
	if (~SPI_MODE & (r_tclk ^ v_tclk)) tx_mclk <= ~r_tclk ^ UCPOL;


	if (RST_TX_INT | WE_ucsra & DI[6]) TXC <= 0;
        else if ( do_txcount & (tx_count == v_count_max) & !tx_data_load)
		 TXC <= 1;
// 	else if (~(tx_data_load | tx_count_en)) TXC <= 1;


	if (OE_udr | (WE_ucsrb & ~DI[4])) RXC   <= 0;
	else if (rx_data_rdy & ~rx_count_en) RXC <= 1;

	if (WE_ucsrb & ~DI[4])
	  begin
	    rx_udr_rdy <= 0;
	    rx_data_rdy <= 0;
	  end
	else if (OE_udr) rx_udr_rdy <= 0;

  	if ( ~SPI_MODE & 
	    ( (UMSEL[0] & do_rx & ~rx_count_en & ~r_rclk & ~v_rxd)
  	    | (~UMSEL[0] & rx_en & ~rx_count_en & ~v_rxd) ) )
	  begin
	    if (rx_data_rdy) DOR <= 1;
	    rx_parity <= 0;
	    rx_ps <= 0;
	    rx_count <= 5'b00000;
	    rx_count_en <= 1;
	    r_count_max_x <= v_bit_num_x;
	  end
        else if (r_ps == UBRR)
	   rx_ps <= rx_ps + 1;

  	if (~UMSEL[0] & ~rx_count_en & ~v_rxd) r_rclk <= 1;
	else r_rclk <= v_rclk;

	if (do_rxcount)
	  begin
	    rx_count <= rx_count + 1;
	    if (~SPI_MODE & (rx_count == v_count_max) )rx_count_en <= 0;
	    if (~SPI_MODE & (rx_count == v_count_max)) rx_data_rdy <= 1;
	    if ( SPI_MODE & (tx_count == v_count_max + 1)) rx_data_rdy <= 1;
	  end
	if (do_rxin)
	  begin
	    rx_c <= v_rxd;
	  end
	if (do_rxsft)
	  begin
	    rx_parity <= rx_parity ^ rx_c;
	    rx_data[6:0] <= rx_data[7:1];
	    rx_data[7] <= (v_bit_num_x == 0) ? rx_c : rx_data[8];
	    rx_data[8] <= (v_bit_num_x == 0) ? 0
	    		: (v_bit_num_x == 1) ? rx_c : rx_data[9];
	    rx_data[9] <= (v_bit_num_x <= 1) ? 0
	    		: (v_bit_num_x == 2) ? rx_c : rx_data[10];
	    rx_data[10] <= (v_bit_num_x == 3) ? rx_c : 0;
	  end

	if ( ~(OE_udr | (WE_ucsrb & ~DI[4])) 
	    & (rx_data_rdy & ~rx_count_en & ~rx_udr_rdy) )
	  begin
	    rx_udr_rdy <= 1;
	    rx_data_rdy <= 0;
	    r_udr_rx[7:0] <= rx_data[7:0];
`ifdef FIXED_UCSZ
	    if(v_bit9) RXB8 <= rx_data[8];
`else
	    RXB8 <= rx_data[8];
`endif
	    if (UPM[1] &
	      (  v_bit7 & (UPM[0]^rx_data[7]^rx_parity^ USBS^rx_data[8])
	       | v_bit8 & (UPM[0]^rx_data[8]^rx_parity^ USBS^rx_data[9]) ))
			UPE <= 1;
	    if ( (v_bit7 & ~UPM[1]) & ~rx_data[7]
	        |(v_bit7 &  UPM[1]) & ~rx_data[8]
	        |(v_bit8 & ~UPM[1]) & ~rx_data[8]
	        |(v_bit8 &  UPM[1]) & ~rx_data[9]
	        |(v_bit9 & ~UPM[1]) & ~rx_data[9] )
			FE <= 1;
	  end

`ifdef FIXED_UBRR
`elsif UBRR_BITS
`else
	if(WE_ubrrl) UBRR[7:0] <= DI;
	if(WE_ubrrh) UBRR[11:8] <= DI[3:0];
`endif

//	if(WE_ucsra) FE    <= DI[4];
//	if(WE_ucsra) DOR   <= DI[3];
//	if(WE_ucsra) UPE   <= DI[2];
	if(WE_ucsra) FE    <= 0;
	if(WE_ucsra) DOR   <= 0;
	if(WE_ucsra) UPE   <= 0;
`ifdef FIXED_U2X
`else
	if(WE_ucsra) U2X   <= DI[1];
`endif
`ifdef FIXED_MPCM
`else
	if(WE_ucsra) MPCM  <= DI[0];
`endif
	if(WE_ucsrb) RXCIE   <= DI[7];
	if(WE_ucsrb) TXCIE   <= DI[6];
	if(WE_ucsrb) UDRIE   <= DI[5];
	if(WE_ucsrb) rx_en    <= DI[4];
	if(WE_ucsrb) tx_en    <= DI[3];
`ifdef FIXED_UCSZ
	if(WE_ucsrb & v_bit9) RXB8    <= DI[1];
	if(WE_ucsrb & v_bit9) TXB8    <= DI[0];
`else
	if(WE_ucsrb) RXB8    <= DI[1];
	if(WE_ucsrb) TXB8    <= DI[0];
`endif

`ifdef FIXED_UMSEL
`else
	if(WE_ucsrc) UMSEL   <= DI[7:6];
`endif
`ifdef FIXED_UPM
`else
	if(WE_ucsrc) UPM     <= DI[5:4];
`endif
`ifdef FIXED_USBS
`else
	if(WE_ucsrc) USBS    <= DI[3];
`endif
`ifdef FIXED_UCPOL
`else
	if(WE_ucsrc) UCPOL   <= DI[0];
`endif

`ifdef FIXED_UCSZ
`else
	if(WE_ucsrc) r_ucsz0  <= DI[1];
	if(WE_ucsrb) r_ucsz2  <= DI[2];
`endif
      end
  end

`ifdef FIXED_UBRR
`elsif UBRR_BITS
  generate 
    genvar i;
    for (i=0; (i < `UBRR_BITS ) & (i < 8) ; i=i+1)
       begin : ubrr_bit_l
	 assign v_ubrr[i] = UBRR[i];
         always @(posedge CLK)
         begin
	   if (RESET == 0) UBRR[i] <= 0;
	   else if(WE_ubrrl) UBRR[i] <= DI[i];
         end
       end
    for (i=8; i < `UBRR_BITS ; i=i+1)
       begin : ubrr_bit_h
	 assign v_ubrr[i] = UBRR[i];
         always @(posedge CLK)
         begin
	   if (RESET == 0) UBRR[i] <= 0;
	   else if(WE_ubrrh) UBRR[i] <= DI[i-8];
         end
       end
    for (i= `UBRR_BITS ; i<16 ; i=i+1)
       begin : ubrr_zero
	 assign v_ubrr[i] = 0;
       end
  endgenerate
`endif
endmodule
