`timescale 10ps / 10ps
`define GTK // use VCD

`include "rtavr_defs.v"

`define MODULE	tb_pw
//`define CLK_LIMIT	300000	// for DEPATH == 8
`define CLK_LIMIT	10000	// for DEPATH == 4
`ifdef QUEEN_CLK_LIMIT
`define CLK_LIMIT `QUEEN_CLK_LIMIT
`endif

module `MODULE;

   reg [23:0] CLK_COUNT;
   reg CLK2X;  // output
   reg RESET;  // output
   wire CLK;   // input
   reg PC2;    // 

  `ifdef RTAVR_PORT_ESCALATION
     wire [23:0] DDR;
     wire [23:0] PORT;
     reg [23:0] PIN;
  `endif

`ifdef PRC_WATCH
   wire [7:0] PW_SREG;
   wire [11:0] PW_SP;
   wire [11:0] PW_PC;
   reg [11:0] PW_PC1;
   reg [11:0] PW_PC2;
   reg [11:0] PW_PC3;
   reg [7:0] PW_SREG_PREV;
   wire [7:0] PW_LOAD_REG;
   wire [3:0] PW_LOAD_ID;
   wire [7:0] PW_STORE_MEM;
   wire [11:0] PW_STORE_ADDR;
`endif

rtavr top (.RESET(RESET), .CLK2X(CLK2X)
		, .CLK(CLK)
  `ifdef RTAVR_PORT_ESCALATION
		, .DDR(DDR[23:0])
		, .PORT(PORT[23:0])
		, .PIN(PIN[23:0])
  `else
    `ifdef IOR_HAVE_PORTA
		, .PORTA(PORTA[7:0])
    `endif
    `ifdef IOR_HAVE_PORTB
		, .PORTB(PORTB[7:0])
    `endif
    `ifdef IOR_HAVE_PORTC
		, .PORTC(PORTC[7:0])
    `endif
  `endif
`ifdef PRC_WATCH
	,  .PW_STORE(PW_STORE)
	,  .PW_LOAD(PW_LOAD)
	,  .PW_SREG(PW_SREG[7:0])
	,  .PW_SP(PW_SP[11:0])
	,  .PW_PC(PW_PC[11:0])
	,  .PW_LOAD_REG(PW_LOAD_REG[7:0])
	,  .PW_LOAD_ID(PW_LOAD_ID[3:0])
	,  .PW_STORE_MEM(PW_STORE_MEM[7:0])
	,  .PW_STORE_ADDR(PW_STORE_ADDR[11:0])
	,  .PW_SLEEP(PW_SLEEP)
	,  .PW_WDR(PW_WDR)
`endif
 );

   parameter CLKP = 5000;        //  50ns
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   always begin
      #CLKH CLK2X = 0;
      #CLKL CLK2X = 1;
   end

   always @(negedge CLK)
   begin
	 if (DDR[16+2]) PIN[16+2] = PORT[16+2]; // PC2 (INT0)
	 if (DDR[16+2]) PC2 = PORT[16+2]; // PC2 (INT0)
   end

`ifdef PRC_WATCH
   integer fp;

   initial begin
      fp = $fopen("tb.log");
   end

   always @(posedge CLK)
   begin
	PW_PC1 <= PW_PC;
	PW_PC2 <= PW_PC1;
	PW_PC3 <= PW_PC2;
	PW_SREG_PREV <= PW_SREG;
     if(PW_LOAD)
	$fdisplay(fp, "L r%2d = %02x : PC %04x FLAGS %02x SP %04x"
		, PW_LOAD_ID+16 , PW_LOAD_REG
		,  { 4'h0 ,  PW_PC2 } , PW_SREG, { 4'h0 ,  PW_SP} );
     else if(PW_STORE)
	$fdisplay(fp, "S mem[%04x] = %02x : PC %04x FLAGS %02x SP %04x"
		, { 4'h0 ,PW_STORE_ADDR} , PW_STORE_MEM
		,  { 4'h0 ,  PW_PC2 } , PW_SREG, { 4'h0 ,  PW_SP} );
     else if( PW_SLEEP )
	$fdisplay(fp, "SLEEP  : PC %04x FLAGS %02x SP %04x"
		,  { 4'h0 ,  PW_PC2 } , PW_SREG, { 4'h0 ,  PW_SP} );
  `ifdef PRC_WATCH_DETAIL
     else if( (PW_SREG != PW_SREG_PREV) || ( PW_PC2 != PW_PC3 + 1))
	$fdisplay(fp, "       : PC %04x FLAGS %02x SP %04x"
		,  { 4'h0 ,  PW_PC2 } , PW_SREG, { 4'h0 ,  PW_SP} );
  `endif
   end
`endif

`ifdef GTK
   initial begin
      $dumpfile("tb.vcd");
      $dumpvars(0,`MODULE.top);
   end
`endif

   initial begin : test
      sys_reset();
      while ( (CLK_COUNT < `CLK_LIMIT) & ~PW_SLEEP )
      begin
         @(posedge CLK2X);
         @(posedge CLK2X);
         CLK_COUNT = CLK_COUNT + 1;
      end
//      @(posedge CLK);
      #1
      $fdisplay(fp, "*** total-clock %d ***", CLK_COUNT);
      $display("*** Simulation Complete! ***");
      $finish;
   end

   task sys_reset;
      begin
         PC2 = 0;
         CLK_COUNT = 0;
         #1 RESET = 0;
         CLK2X = 0;
         @(posedge CLK2X);
         @(posedge CLK2X);
	 @(negedge CLK2X);
         @(posedge CLK2X);
         #1 RESET = 1;
         @(posedge CLK2X);
         @(posedge CLK2X);
         CLK_COUNT = CLK_COUNT + 1;
      end
   endtask // sys_reset
   
endmodule // test
